@@ -1,14 +1,26 @@
 /*
  * Miscellaneous routines for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
+/*
+ * This is called when the maximum expression size has been exceeded.
+ *
+ * There is no return.
+ */
+void
+error_huge()
+{
+	longjmp(jmp_save, 14);
+}
+
 /*
  * Check if a floating point math function flagged an error.
  */
+void
 check_err()
 {
 	switch (errno) {
@@ -29,26 +41,6 @@ check_err()
 	}
 }
 
-/*
- * Erase all equation spaces.
- * Similar to a restart.
- */
-clear_all()
-{
-	int	i;
-
-	cur_equation = 0;
-	for (i = 0; i < n_equations; i++) {
-		n_lhs[i] = 0;
-		n_rhs[i] = 0;
-	}
-	for (i = 0; var_names[i]; i++) {
-		free(var_names[i]);
-		var_names[i] = NULL;
-	}
-	clear_sign_array();
-}
-
 /*
  * Allocate the needed temp storage.
  *
@@ -72,6 +64,7 @@ init_mem()
 /*
  * Initialize the global variables.
  */
+void
 init_gvars()
 {
 	domain_check = false;
@@ -94,6 +87,7 @@ init_gvars()
  * Clean up when processing is unexpectedly
  * interrupted or terminated.
  */
+void
 clean_up()
 {
 	init_gvars();
@@ -108,6 +102,7 @@ clean_up()
 /*
  * Restart sign array.
  */
+void
 clear_sign_array()
 {
 	int	i;
@@ -120,6 +115,7 @@ clear_sign_array()
 /*
  * Register all sign variables.
  */
+void
 set_sign_array()
 {
 	int	i, j;
@@ -168,10 +164,36 @@ long	*vp;
 	return true;
 }
 
+/*
+ * Erase all equation spaces.
+ * Similar to a restart.
+ */
+void
+clear_all()
+{
+	int	i;
+
+/* set to equation space number 1 */
+	cur_equation = 0;
+/* erase all equation spaces */
+	for (i = 0; i < n_equations; i++) {
+		n_lhs[i] = 0;
+		n_rhs[i] = 0;
+	}
+/* forget all variables names */
+	for (i = 0; var_names[i]; i++) {
+		free(var_names[i]);
+		var_names[i] = NULL;
+	}
+/* reset everything to be in a known state */
+	clear_sign_array();
+	init_gvars();
+}
+
 /*
  * Allocate or reuse an empty equation space.
  *
- * Returns new equation space number or -1 on error.
+ * Returns empty equation space number ready for use or -1 on error.
  */
 int
 alloc_next_espace()
@@ -216,7 +238,8 @@ next_espace()
 	if (i < 0) {
 		error(_("Out of free equation spaces."));
 #if	!SILENT
-		printf(_("Please use the clear command on unnecessary equations and try again.\n"));
+		printf(_("Use the clear command on unnecessary equations or\n"));
+		printf(_("restart with \"clear all\" and try again.\n"));
 #endif
 		longjmp(jmp_save, 3);
 	}
@@ -226,6 +249,7 @@ next_espace()
 /*
  * Copy equation number "src" to equation number "dest".
  */
+void
 copy_espace(src, dest)
 int	src, dest;
 {
@@ -255,15 +279,15 @@ int	i;
  * Return true if variable "v" exists in expression.
  */
 int
-found_var(equation, n, v)
-token_type	*equation;
+found_var(p1, n, v)
+token_type	*p1;
 int		n;
 long		v;
 {
 	int	j;
 
 	for (j = 0; j < n; j += 2) {
-		if (equation[j].kind == VARIABLE && equation[j].token.variable == v) {
+		if (p1[j].kind == VARIABLE && p1[j].token.variable == v) {
 			return true;
 		}
 	}
@@ -290,6 +314,7 @@ long	v;
 /*
  * Substitute every instance of "v" in "equation" with "expression".
  */
+void
 subst_var_with_exp(equation, np, expression, len, v)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to equation side length */
@@ -316,11 +341,11 @@ long		v;		/* variable to substitute with expression */
 }
 
 /*
- * Return the minimum level encountered in a sub-expression.
+ * Return the minimum level encountered in a Mathomatic "expression".
  */
 int
-min_level(equation, n)
-token_type	*equation;	/* expression pointer */
+min_level(expression, n)
+token_type	*expression;	/* expression pointer */
 int		n;		/* expression length */
 {
 	int		min1;
@@ -330,28 +355,18 @@ int		n;		/* expression length */
 		if (n <= 0) {
 			return 1;
 		} else {
-			return equation[0].level;
+			return expression[0].level;
 		}
 	}
-	min1 = equation[1].level;
-	ep = &equation[n];
-	for (p1 = &equation[3]; p1 < ep; p1 += 2) {
+	min1 = expression[1].level;
+	ep = &expression[n];
+	for (p1 = &expression[3]; p1 < ep; p1 += 2) {
 		if (p1->level < min1)
 			min1 = p1->level;
 	}
 	return min1;
 }
 
-/*
- * This is called when the maximum expression size has been exceeded.
- *
- * There is no return.
- */
-error_huge()
-{
-	longjmp(jmp_save, 14);
-}
-
 /*
  * Get default equation number from a command parameter string.
  * The equation number must be the only parameter.
@@ -456,7 +471,7 @@ int		*np;		/* pointer to the returned parsed expression length */
 }
 
 /*
- * Prompt for a variable from the user.
+ * Prompt for a variable name from the user.
  *
  * Return true if successful.
  */
@@ -495,7 +510,7 @@ int	i;	/* equation space number */
 		return true;
 	}
 	if (n_lhs[i] <= 0) {
-		error(_("Equation space empty."));
+		error(_("Equation space is empty."));
 		return true;
 	}
 	return false;
@@ -520,10 +535,9 @@ current_not_defined()
 }
 
 /*
- * Common routine to output the prompt in prompt_str[]
- * and get a line of input from stdin.
- * All Mathomatic input comes from this routine,
- * except for file reading.
+ * Common routine to output the prompt in prompt_str[] and get a line of input from stdin.
+ * All Mathomatic input comes from this routine, except for file reading.
+ * If there is no more input (EOF), exit this program with no error.
  *
  * Returns "string" if successful or NULL on error.
  */
@@ -536,6 +550,7 @@ int	n;		/* maximum size of "string" in bytes */
 	error(_("Missing command line argument."));
 	return NULL;
 #else
+	int	i;
 #if	READLINE
 	char	*cp;
 #endif
@@ -547,8 +562,10 @@ int	n;		/* maximum size of "string" in bytes */
 #if	READLINE
 	if (readline_enabled) {
 		cp = readline(prompt_str);
-		if (cp == NULL)
+		if (cp == NULL) {
+			fprintf(gfp, _("\nEnd of input.\n"));
 			exit_program(0);
+		}
 		my_strlcpy(string, cp, n);
 		if (string[0]) {
 			add_history(cp);
@@ -557,21 +574,32 @@ int	n;		/* maximum size of "string" in bytes */
 		}
 	} else {
 		printf("%s", prompt_str);
-		if (fgets(string, n, stdin) == NULL)
+		if (fgets(string, n, stdin) == NULL) {
+			fprintf(gfp, _("\nEnd of input.\n"));
 			exit_program(0);
+		}
 	}
 #else
 	printf("%s", prompt_str);
-	if (fgets(string, n, stdin) == NULL)
+	if (fgets(string, n, stdin) == NULL) {
+		fprintf(gfp, _("\nEnd of input.\n"));
 		exit_program(0);
+	}
 #endif
+	i = strlen(string) - 1;
+	if (i >= 0 && string[i] == '\n') {
+		string[i] = '\0';
+	}
+	if (gfp != stdout) {
+		fprintf(gfp, "%s%s\n", prompt_str, string);
+	}
 	set_error_level(string);
 	return string;
 #endif
 }
 
 /*
- * Display the prompt in prompt_str[] and wait for "y" or "n".
+ * Display the prompt in prompt_str[] and wait for "y" or "n" followed by Enter.
  *
  * Return true if "y".
  */
@@ -612,11 +640,12 @@ int	en;	/* equation space number */
 		if (factor_int_flag) {
 			factor_int_sub(en);
 		}
+		free(result_str);
 		result_str = list_equation(en, false);
 		return(result_str != NULL);
 	}
 #endif
-	return list_sub(en);
+	return(list_sub(en) != 0);
 }
 
 /*
@@ -630,8 +659,9 @@ char	*cp;
 }
 
 /*
- * Process an equation number range given in text string *cpp.
- * Skip past all spaces and update *cpp to point to the next argument.
+ * Process an equation number range given in text string "*cpp".
+ * Skip past all spaces and update "*cpp" to point to the next argument.
+ * If no equation number or range is given, assume the current equation is wanted.
  *
  * Return true if successful,
  * with the starting equation number in "*ip"
@@ -872,7 +902,7 @@ int		n1;	/* expression length */
 }
 
 /*
- * Set *vp if single variable expression.
+ * Set "*vp" if single variable expression.
  *
  * Return true if expression contains no variables.
  */
@@ -907,7 +937,7 @@ long		*vp;		/* variable pointer */
 }
 
 /*
- * Return true if expression contains infinity or nan.
+ * Return true if expression contains infinity or NaN (Not a Number).
  */
 int
 exp_contains_infinity(p1, n1)
@@ -923,3 +953,40 @@ int		n1;	/* expression length */
 	}
 	return false;
 }
+
+/*
+ * Return true if expression contains NaN (Not a Number).
+ */
+int
+exp_contains_nan(p1, n1)
+token_type	*p1;	/* expression pointer */
+int		n1;	/* expression length */
+{
+	int	i;
+
+	for (i = 0; i < n1; i += 2) {
+		if (p1[i].kind == CONSTANT && isnan(p1[i].token.constant)) {
+			return true;
+		}
+	}
+	return false;
+}
+
+/*
+ * Return true if expression is numeric (not symbolic).
+ * Pseudo-variables e, pi, i, and sign are considered numeric.
+ */
+int
+exp_is_numeric(p1, n1)
+token_type	*p1;	/* expression pointer */
+int		n1;	/* expression length */
+{
+	int	i;
+
+	for (i = 0; i < n1; i += 2) {
+		if (p1[i].kind == VARIABLE && (p1[i].token.variable & VAR_MASK) > SIGN) {
+			return false;		/* not numerical (contains a variable) */
+		}
+	}
+	return true;
+}
