@@ -1,7 +1,7 @@
 /*
  * Main include file for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #define	true	1
@@ -12,15 +12,17 @@
 #endif
 
 #ifndef	max
-#define max(a, b)    (((a) > (b)) ? (a) : (b))	/* return the maximum of two values */
+#define max(a, b)    (((a) > (b)) ? (a) : (b))		/* return the maximum of two values */
 #endif
-
 #ifndef	min
-#define min(a, b)    (((a) < (b)) ? (a) : (b))	/* return the minimum of two values */
+#define min(a, b)    (((a) < (b)) ? (a) : (b))		/* return the minimum of two values */
 #endif
 
+#ifndef	INFINITY
+#define INFINITY	HUGE_VAL			/* the floating point, positive infinity constant */
+#endif
 #ifndef	isfinite
-#define	isfinite(d)	finite(d)
+#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity or NaN) */
 #endif
 
 #define	ARR_CNT(a)	(sizeof(a)/sizeof(a[0]))	/* returns the number of elements in an array */
@@ -31,7 +33,7 @@
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function */
 #define always_positive(power)	(fmod((double) (power), 2.0) == 0.0)	/* true if all real numbers raised to "power" result in positive, real numbers */
 
-#if	I18N	/* internationalization, translators needed! */
+#if	I18N	/* internationalization */
 #define _(str)		gettext(str)
 #else
 #define _(str)		str
@@ -43,7 +45,7 @@
 #define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temp file template */
 
 #define	PROMPT		"-> "				/* user interface prompt string */
-#define	HTML_PROMPT	"-&gt; "
+#define	HTML_PROMPT	"&mdash;&gt; "
 
 #define	MAX_CMD_LEN	PATH_MAX			/* maximum command line length (not equations), also max file name length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
@@ -61,8 +63,9 @@
 
 /*
  * The following defines the default maximum mathematical expression size.
- * Expression arrays are allocated with this size by default.
- * It is linearly related to the actual memory usage of Mathomatic.
+ * Expression arrays are allocated with this size by default,
+ * there are 2 of these for each equation space (1 for LHS and 1 for RHS).
+ * DEFAULT_N_TOKENS is linearly related to the actual memory usage of Mathomatic.
  * This should be made much smaller for Palmtops or embedded systems.
  * Do not set to less than 100.
  */
@@ -70,10 +73,10 @@
 
 #define	DIVISOR_SIZE	(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
 
-#define	MAX_VAR_NAMES	8000			/* maximum number of long variable names, keep this under (VAR_MASK - VAR_OFFSET) */
-#define	MAX_VAR_LEN	100			/* maximum size of long variable names */
+#define	MAX_VAR_NAMES	8000			/* maximum number of variable names, keep this under (VAR_MASK - VAR_OFFSET) */
+#define	MAX_VAR_LEN	100			/* maximum number of characters in variable names */
 
-#define	MAX_VARS	500			/* maximum number of unique variables handled in each equation */
+#define	MAX_VARS	min(DEFAULT_N_TOKENS / 4, 1000)	/* maximum number of unique variables handled in each equation */
 
 #define	VAR_OFFSET	'A'			/* makes space for predefined variables */
 #define	VAR_MASK	0x3fffL			/* mask for bits containing a reference to the variable name */
@@ -95,7 +98,7 @@ typedef union {
 /* predefined special variables follow (order is important) */
 #define	V_NULL		0L	/* null variable (display nothing) */
 #define	V_E		1L	/* the constant "e" or "e#" */
-#define	V_PI		2L	/* the constant "pi" */
+#define	V_PI		2L	/* the constant "pi" or "pi#" */
 #define	IMAGINARY	3L	/* the imaginary constant "i" or "i#" */
 #define	SIGN		4L	/* for "sign" variables */
 #define	MATCH_ANY	5L	/* match any variable (wild-card variable) */
@@ -122,6 +125,7 @@ typedef struct {		/* variable sort data structure */
 	int	count;		/* number of times the variable occurs */
 } sort_type;
 
+/* Debugging and error reporting macros: */
 #if	SILENT
 #define	error(str)			{ error_str = str; }	/* must be a constant string, temp strings don't work */
 #define	list_tdebug(level)
