@@ -1,7 +1,7 @@
 /*
  * Mathomatic differentiation routines and commands.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -287,8 +287,8 @@ int
 derivative_cmd(cp)
 char	*cp;
 {
-	long		v = 0;
 	int		i;
+	long		v = 0;
 	long		l1, order = 1;
 	token_type	*source, *dest;
 	int		n1, *nps, *np;
@@ -314,7 +314,7 @@ char	*cp;
 			cp = skip_param(cp);
 			v = MATCH_ANY;
 		} else {
-			if (!isdigit(*cp)) {
+			if (isvarchar(*cp)) {
 				cp = parse_var2(&v, cp);
 				if (cp == NULL) {
 					return false;
@@ -344,11 +344,10 @@ char	*cp;
 	}
 #if	!SILENT
 	list_var(v, 0);
-	printf(_("Differentiating with respect to (%s) and simplifying"), var_str);
 	if (n_rhs[cur_equation]) {
-		printf(_(" the RHS...\n"));
+		printf(_("Differentiating the RHS with respect to (%s) and simplifying...\n"), var_str);
 	} else {
-		printf("...\n");
+		printf(_("Differentiating with respect to (%s) and simplifying...\n"), var_str);
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -359,7 +358,7 @@ char	*cp;
 			error(_("Differentiation failed."));
 			return false;
 		}
-		simpa_side(dest, &n1, true);
+		simpa_side(dest, &n1, true, false);
 	}
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
@@ -379,6 +378,7 @@ char	*cp;
 {
 	int		i;
 	long		v = 0;
+	long		l1, order = 1;
 	token_type	want;
 	token_type	*source;
 	int		n;
@@ -386,6 +386,7 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	i = next_espace();
 	if (n_rhs[cur_equation]) {
 		if (!solved_equation(cur_equation)) {
 			error(_("The current equation is not solved for a variable."));
@@ -398,11 +399,17 @@ char	*cp;
 		n = n_lhs[cur_equation];
 	}
 	if (*cp) {
-		cp = parse_var2(&v, cp);
-		if (cp == NULL) {
-			return false;
+		if (isvarchar(*cp)) {
+			cp = parse_var2(&v, cp);
+			if (cp == NULL) {
+				return false;
+			}
 		}
-		if (extra_characters(cp)) {
+		if (*cp) {
+			order = decstrtol(cp, &cp);
+		}
+		if (*cp || order <= 0) {
+			error(_("The order must be a positive integer."));
 			return false;
 		}
 	}
@@ -419,16 +426,17 @@ char	*cp;
 		error(_("Variable not found."));
 		return false;
 	}
-	i = next_espace();
 	blt(rhs[i], source, n * sizeof(token_type));
-/* take the derivative with respect to the specified variable and simplify: */
-	if (!differentiate(rhs[i], &n, v)) {
-		error(_("Differentiation failed."));
-		return false;
+/* take derivatives with respect to the specified variable and simplify: */
+	for (l1 = 0; l1 < order; l1++) {
+		if (!differentiate(rhs[i], &n, v)) {
+			error(_("Differentiation failed."));
+			return false;
+		}
+		simpa_side(rhs[i], &n, true, false);
 	}
-	simpa_side(rhs[i], &n, true);
 	if (!found_var(rhs[i], n, v)) {
-		error(_("There are no extrema for this equation."));
+		error(_("There are no solutions."));
 		return false;
 	}
 	n_rhs[i] = n;
@@ -444,7 +452,7 @@ char	*cp;
 		error(_("Solve failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], false);
+	simpa_side(rhs[i], &n_rhs[i], false, false);
 	return return_result(cur_equation);
 }
 
@@ -465,6 +473,7 @@ char	*cp;
 	int		our_nlhs, our_nrhs;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
+	int		flag = false;
 
 	cp_start = cp;
 	if (current_not_defined()) {
@@ -491,7 +500,7 @@ char	*cp;
 		dest = lhs[i];
 		np = &n_lhs[i];
 	}
-	if (*cp && !isdigit(*cp)) {
+	if (*cp && isvarchar(*cp)) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
@@ -537,7 +546,7 @@ char	*cp;
 	} else {
 #if	!SILENT
 		list_var(v, 0);
-		printf(_("Taylor series expansion about %s = point.\n"), var_str);
+		printf(_("Taylor series expansion around %s = point.\n"), var_str);
 #endif
 		my_strlcpy(prompt_str, _("Enter point: "), sizeof(prompt_str));
 		if (!get_expr(lhs[our], &our_nlhs)) {
@@ -622,8 +631,12 @@ loop_again:
 	n1 += 4;
 	for (; i1 < n1; i1++)
 		dest[i1].level++;
-	simp_loop(dest, &n1);
-/*	side_debug(1, dest, n1); */
+	uf_simp(dest, &n1);
+	if (!flag && exp_contains_infinity(dest, n1)) {
+		debug_string(0, _("Singularity warning: result invalid because it contains infinity or NaN."));
+		flag = true;
+	}
+	side_debug(1, dest, n1);
 	if (n < order) {
 		if (n > 0) {
 			if (!differentiate(rhs[our], &our_nrhs, v)) {
@@ -631,7 +644,7 @@ loop_again:
 				return false;
 			}
 		}
-		simpa_side(rhs[our], &our_nrhs, true);
+		simpa_side(rhs[our], &our_nrhs, true, true);
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
 			i1 = n1;
 			if ((i1 + 1 + our_nrhs) > n_tokens)
@@ -677,6 +690,7 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	i = next_espace();
 	if (n_rhs[cur_equation] == 0) {
 /* make expression into an equation: */
 		blt(rhs[cur_equation], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
@@ -729,14 +743,13 @@ char	*cp;
 	if (extra_characters(cp))
 		return false;
 /* copy to a new equation space and work on the copy: */
-	i = next_espace();
 	copy_espace(cur_equation, i);
 	cur_equation = i;
 /* see if the limit expression contains infinity: */
 	simp_loop(tes, &n_tes);
 	infinity_flag = exp_contains_infinity(tes, n_tes);
 /* first fully simplify the current equation and group denominators together: */
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
 	group_proc(rhs[cur_equation], &n_rhs[cur_equation]);
 	found_den = complex_den = false;
 	for (j = 1; j < n_rhs[cur_equation]; j += 2) {
@@ -778,12 +791,12 @@ char	*cp;
 	}
 	if (infinity_flag ? (!complex_den && found_den) : (!found_den)) {
 /* in this case, just substitute the limit variable with the limit expression and simplify: */
+		debug_string(0, "Substituting...");
 		subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], tes, n_tes, v);
-		simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
-		return return_result(cur_equation);
+		goto finish_up;
 	}
-	debug_string(1, "Taking the limit by solving...");
-	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == HUGE_VAL) {
+	debug_string(0, "Solving...");
+	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
 		n_tes = 1;
@@ -811,16 +824,25 @@ char	*cp;
 	n_lhs[cur_equation] = n_tes;
 /* symbolically simplify the RHS: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
 	symb_flag = false;
+	if (exp_contains_nan(rhs[cur_equation], n_rhs[cur_equation])) {
+		error(_("Unable to take limit; result contains NaN (Not a Number)."));
+		return false;
+	}
 /* solve back for the original variable: */
 	if (solve_sub(&solved_v, 1, lhs[cur_equation], &n_lhs[cur_equation], rhs[cur_equation], &n_rhs[cur_equation]) <= 0) {
 		error(_("Can't take the limit because solve failed."));
 		return false;
 	}
+finish_up:
 /* symbolically simplify before returning the result: */
 	symb_flag = true;
-	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false);
+	simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
 	symb_flag = false;
+	if (exp_contains_nan(rhs[cur_equation], n_rhs[cur_equation])) {
+		error(_("Unable to take limit; result contains NaN (Not a Number)."));
+		return false;
+	}
 	return return_result(cur_equation);
 }
