@@ -1,7 +1,7 @@
 /*
  * General floating point GCD routine and associated code for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -11,10 +11,10 @@
  * by using the Euclidean GCD algorithm.
  *
  * The GCD is defined as the largest positive number that evenly divides both "d1" and "d2".
- * Will work with non-integers, but there may be some floating point error.
- * Always works correctly with integers.
+ * Will usually work with non-integers, but there may be some floating point error.
+ * Always works perfectly with integers up to MAX_K_INTEGER.
  *
- * Return 0.0 on failure, otherwise return the positive GCD.
+ * Return 0 on failure or if either parameter is 0, otherwise return the positive GCD.
  */
 double
 gcd(d1, d2)
@@ -58,6 +58,27 @@ double	d1, d2;
 	return 0.0;
 }
 
+/*
+ * Return the verified exact Greatest Common Divisor (GCD) of doubles "d1" and "d2".
+ *
+ * Return 0 on failure or if either parameter is 0, otherwise return the positive GCD.
+ */
+double
+gcd_verified(d1, d2)
+double	d1, d2;
+{
+	double	divisor;
+
+	divisor = gcd(d1, d2);
+	if (divisor != 0.0) {
+		if (fmod(d1, divisor) != 0.0)
+			return 0.0;
+		if (fmod(d2, divisor) != 0.0)
+			return 0.0;
+	}
+	return divisor;
+}
+
 /*
  * Return a floating point double rounded to the nearest integer.
  */
@@ -151,6 +172,7 @@ double	*denominatorp;	/* returned denominator */
 /*
  * Convert non-integer constants in an equation side to fractions, when appropriate.
  */
+void
 make_fractions(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
