@@ -1,14 +1,16 @@
 /*
  * Mathomatic global variables and arrays.
+ * C initializes global variables and arrays to zero by default.
+ * This is required for proper operation.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  *
  * Most global variables for Mathomatic are defined here and duplicated in "externs.h".
  */
 
 #include "includes.h"
 
-int		n_tokens = DEFAULT_N_TOKENS;	/* maximum size of expressions */
+int		n_tokens = DEFAULT_N_TOKENS;	/* maximum size of expressions, do not change without a restart */
 int		n_equations;			/* number of equation spaces allocated */
 int		cur_equation;			/* current equation space number (origin 0) */
 
@@ -38,13 +40,14 @@ int		factor_int_flag;			/* factor integers when displaying expressions */
 int		display2d = true;			/* "set display2d" flag */
 int		preserve_roots = true;			/* set option to preserve roots like (2^.5) */
 int		true_modulus;				/* true for mathematically correct modulus */
-int		screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width of the terminal; 0 = infinite */
-int		screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the terminal; 0 = infinite */
+volatile int	screen_columns = STANDARD_SCREEN_COLUMNS;	/* screen width of the terminal; 0 = infinite */
+volatile int	screen_rows = STANDARD_SCREEN_ROWS;		/* screen height of the terminal; 0 = infinite */
 int		finance_option;				/* for displaying dollars and cents */
 int		autosolve = true;			/* Allows solving by typing the variable name at the prompt */
 int		autocalc = true;			/* Allows automatically calculating a numerical expression */
 char		special_variable_character = '\\';	/* user defined special character for variable names */
-int		e_i_as_constants = true;		/* set true to allow "e" and "i" as special constants */
+int		integer_coefficients = true;		/* if true, prefer displaying all integer coefficients */
+int		right_associative_power;		/* if true, evaluate power operators right to left */
 #if	!SILENT
 int		debug_level;				/* current debug level */
 #endif
@@ -59,11 +62,11 @@ double		small_epsilon	= 0.000000000000005;	/* for small round-off errors */
 double		epsilon		= 0.00000000000005;	/* for larger, accumulated round-off errors */
 
 /* string variables */
-char		*var_names[MAX_VAR_NAMES];	/* storage for long variable names */
-char		var_str[MAX_VAR_LEN+80];	/* temp storage for variable names */
-char		prompt_str[MAX_PROMPT_LEN];	/* temp storage for prompt strings */
+char		*var_names[MAX_VAR_NAMES];	/* index for storage of variable name strings */
+char		var_str[MAX_VAR_LEN+80];	/* temp storage for listing a variable name */
+char		prompt_str[MAX_PROMPT_LEN];	/* temp storage for the prompt string */
 #if	CYGWIN
-char		*dir_path;			/* directory path to the executable */
+char		*dir_path;			/* directory path to the executable, only set and used in CYGWIN version */
 #endif
 char		*prog_name = "mathomatic";	/* name of this program */
 
@@ -78,7 +81,7 @@ FILE		*gfp;			/* global output file pointer */
 jmp_buf		jmp_save;		/* for setjmp() when an error is encountered */
 int		test_mode;		/* test mode flag (-t) */
 int		quiet_mode;		/* quiet mode (don't display prompts) */
-int		html_flag;		/* true for HTML mode */
+int		html_flag;		/* true for HTML output mode */
 int		readline_enabled = true;
 int		partial_flag;		/* true for partial unfactoring of algebraic fractions */
 int		symb_flag;		/* true for "simplify symbolic" */
@@ -86,7 +89,7 @@ int		high_prec;		/* flag to output constants in higher precision (used when savi
 int		input_column;		/* current column number on the screen at the beginning of a parse */
 int		sign_flag;		/* true when all "sign" variables are to compare equal */
 int		domain_check;		/* flag to track domain errors in the pow() function */
-int		approximate_roots;	/* true if in calculate command (force approximations of roots like (2^.5)) */
+int		approximate_roots;	/* true if in calculate command (force approximation of roots like (2^.5)) */
 
 /* library variables go here */
 #if	SILENT
