@@ -3,7 +3,7 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,7 +12,7 @@
  * The following structure is used for each Mathomatic command.
  */
 typedef	struct {
-	char	*name;			/* command name to be typed by user */
+	char	*name;			/* command name to be typed by user (must not contain any spaces) */
 	char	*secondary_name;	/* another name for this command */
 	int	(*func)();		/* function that handles this command */
 					/* function is passed a char pointer and returns true if successful */
@@ -24,69 +24,74 @@ typedef	struct {
  * The Mathomatic command table follows.  It should be in alphabetical order.
  */
 static com_type com_list[] = {
-/*	command name,	alternate name,		function,	usage,							information */
+/*	command name,	alternate name,		function,		usage,							information */
+{	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces." },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,	"[variable number-of-iterations]",			"Temporarily plug in values for variables and approximate." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"repeat\"] [\"factor\"] [variable number-of-iterations]",	"Temporarily plug in values for variables and approximate." },
 #endif
-{	"clear",	NULL,			clear_cmd,	"[equation-numbers-or-ranges]",				"Delete expressions or equations so equation spaces can be reused." },
+{	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused." },
 #if	!LIBRARY
-{	"code",		NULL,			code_cmd,	"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-range]",	"Output C, Java, or Python code for the specified equations." },
+{	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations." },
 #endif
-{	"compare",	NULL,			compare_cmd,	"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
-{	"copy",		NULL,			copy_cmd,	"[equation-number-range]",				"Duplicate the specified equation spaces." },
-{	"derivative",	"differentiate",	derivative_cmd,	"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"compare",	NULL,			compare_cmd,		"equation-number [\"with\" equation-number]",		"Compare two equation spaces to see if they are the same." },
+{	"copy",		NULL,			copy_cmd,		"[equation-number-range]",				"Duplicate the specified equation spaces." },
+{	"derivative",	"differentiate",	derivative_cmd,		"[variable or \"all\"] [order]",			"Symbolically differentiate and simplify, order times." },
+{	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-range]",			"Display equation spaces in multi-line fraction format." },
 #if	!LIBRARY
-{	"divide",	NULL,			divide_cmd,	"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
-#if	(UNIX || CYGWIN) && !SECURE
-{	"edit",		NULL,			edit_cmd,	"[file-name]",						"Edit all equation spaces or an input file." },
+{	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 polynomials/numbers and divide.  Display result and GCD." },
 #endif
+{	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
+#if	!LIBRARY && (UNIX || CYGWIN) && !SECURE
+{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file." },
 #endif
-{	"eliminate",	NULL,			eliminate_cmd,	"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
-{	"extrema",	"extremum",		extrema_cmd,	"[variable]",						"Find the minimums and maximums of the current expression." },
-{	"factor",	NULL,			factor_cmd,	"[\"number\" [integers]] or [equation-number-range] [variables]",	"Trivially factor integers or stored expressions." },
-{	"flist",	"display",		flist_cmd,	"[\"factor\"] [equation-number-range]",			"Display stored expressions in multi-line fraction format." },
-{	"fraction",	NULL,			fraction_cmd,	"[equation-number-range]",				"Convert expressions with any algebraic fractions into a single fraction." },
-{	"help",		NULL,			help_cmd,	"[topic or command-name]",				"Short, built-in help." },
-{	"imaginary",	NULL,			imaginary_cmd,	"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
-{	"integrate",	"integral",		integrate_cmd,	"[\"definite\"] variable [order]",			"Symbolically integrate polynomials, order times." },
-{	"laplace",	NULL,			laplace_cmd,	"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
-{	"limit",	NULL,			limit_cmd,	"variable expression",					"Take the limit of the current expression as variable goes to expression." },
-{	"list",		NULL,			list_cmd,	"[\"export\" or \"maxima\"] [equation-number-range]",	"Display stored expressions in single line format." },
+{	"eliminate",	NULL,			eliminate_cmd,		"[\"repeat\"] variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations." },
+{	"extrema",	"stationary",		extrema_cmd,		"[variable] [order]",					"Find possible local minimums and maximums of the current expression." },
+{	"factor",	NULL,			factor_cmd,		"[\"number\" [integers]] or [equation-number-range] [variables]",	"Factor given integers or factor variables in expressions." },
+{	"fraction",	NULL,			fraction_cmd,		"[equation-number-range]",				"Convert expressions with algebraic fractions into a single fraction." },
+{	"help",		NULL,			help_cmd,		"[topic or command-names]",				"Short, built-in help." },
+{	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of an expression (see the \"real\" command)." },
+{	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials, order times." },
+{	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials." },
+{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\"] [equation-number-ranges]",	"Display equation spaces in single line format." },
 #if	!LIBRARY
-{	"nintegrate",	NULL,			nintegrate_cmd,	"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
-{	"optimize",	NULL,			optimize_cmd,	"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
+{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Approximate the definite integral using Simpson's rule."},
+{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up an equation into smaller multiple equations." },
 #endif
 #if	!LIBRARY
-{	"pause",	NULL,			pause_cmd,	"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
+{	"pause",	NULL,			pause_cmd,		"[text]",						"Wait for user to press the Enter key.  Optionally display a message." },
 #endif
-{	"product",	NULL,			product_cmd,	"variable start end [step]",				"Compute the product as variable goes from start to end." },
+{	"product",	NULL,			product_cmd,		"variable start end [step]",				"Compute the product as variable goes from start to end." },
 #if	READLINE
-{	"push",		NULL,			push_cmd,	"[equation-number-range]",				"Push stored expressions into the readline history." },
+{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push specified equation spaces into the readline history." },
 #endif
 #if	!LIBRARY
-{	"quit",		"exit",			quit_cmd,	"",							"Terminate this program without saving." },
-#endif
-#if	!SECURE && !LIBRARY
-{	"read",		NULL,			read_cmd,	"file-name",						"Read in a text file as if it was typed in." },
-#endif
-{	"real",		NULL,			real_cmd,	"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
-{	"replace",	NULL,			replace_cmd,	"[\"constants\"] or [variables [\"with\" expression]]",	"Substitute variables in the current equation with expressions." },
-#if	!LIBRARY
-{	"roots",	NULL,			roots_cmd,	"root real-part imaginary-part",			"Display all the roots of a complex number." },
+{	"quit",		"exit",			quit_cmd,		"",							"Terminate this program without saving." },
 #endif
 #if	!SECURE
-{	"save",		NULL,			save_cmd,	"file-name",						"Save all stored expressions and equations in a text file." },
+{	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in." },
 #endif
-{	"set",		NULL,			set_cmd,	"[[\"no\"] options]",					"Set or display various session options." },
-{	"simplify",	NULL,			simplify_cmd,	"[\"symbolic\"] [\"quick\"] [equation-number-range]",	"Completely simplify stored expressions." },
-{	"solve",	NULL,			solve_cmd,	"variable or \"0\"",					"Solve the current equation for a variable or for 0." },
-{	"sum",		NULL,			sum_cmd,	"variable start end [step]",				"Compute the summation as variable goes from start to end." },
+{	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of an expression (see the \"imaginary\" command)." },
+{	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
 #if	!LIBRARY
-{	"tally",	NULL,			tally_cmd,	"",							"Prompt for and add numerical entries, show running total and average." },
+{	"roots",	NULL,			roots_cmd,		"root real-part imaginary-part",			"Display all the roots of a complex number." },
 #endif
-{	"taylor",	NULL,			taylor_cmd,	"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,	"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) stored expressions." },
-{	"version",	NULL,			version_cmd,	"",							"Display version number and compile flags used." }
+#if	!SECURE
+{	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file." },
+#endif
+{	"set",		NULL,			set_cmd,		"[[\"no\"] option]",					"Set or display various session options." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"symbolic\"] [\"quick\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions." },
+{	"solve",	NULL,			solve_cmd,		"variable or \"0\"",					"Solve the current equation for a variable or for zero." },
+{	"sum",		NULL,			sum_cmd,		"variable start end [step]",				"Compute the summation as variable goes from start to end." },
+#if	!LIBRARY
+{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show running (grand) total." },
+#endif
+{	"taylor",	NULL,			taylor_cmd,		"[variable] [order] [point]",				"Compute the Taylor series expansion of the current expression." },
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"fully\"] [equation-number-range]",			"Algebraically expand (multiply out) expressions." },
+#if	!LIBRARY
+{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Output all variable names within the specified equations." },
+#endif
+{	"version",	NULL,			version_cmd,		"",							"Display version number, compile flags, and maximum memory usage." }
 };
 
 /*
@@ -102,7 +107,7 @@ parse(n, cp)
 int	n;
 char	*cp;
 {
-	if ((cp = parse_equation(n, cp)) != NULL) {
+	if (parse_equation(n, cp)) {
 		if (n_lhs[n] == 0 && n_rhs[n] == 0)
 			return true;
 		if (n_lhs[n] == 0) {
@@ -131,13 +136,23 @@ process_parse(n, cp)
 int	n;
 char	*cp;
 {
-#if	!LIBRARY
 	int	i;
-#endif
 
-	if ((cp = parse_equation(n, cp)) != NULL) {
-		if (n_lhs[n] == 0 && n_rhs[n] == 0)
+	if (parse_equation(n, cp)) {
+		if (n_lhs[n] == 0 && n_rhs[n] == 0) {
+			if (strcmp(cp, "=") == 0 && n_lhs[cur_equation] > 0 && n_rhs[cur_equation] > 0) {
+				debug_string(0, _("Swapping equation sides of the current equation..."));
+				n = cur_equation;
+				i = n_lhs[n];
+				blt(tes, lhs[n], n_lhs[n] * sizeof(token_type));
+				n_lhs[n] = n_rhs[n];
+				blt(lhs[n], rhs[n], n_rhs[n] * sizeof(token_type));
+				n_rhs[n] = i;
+				blt(rhs[n], tes, i * sizeof(token_type));
+				return return_result(cur_equation);
+			}
 			return true;
+		}
 		if (n_lhs[n] == 0 || n_rhs[n] == 0) {
 			if (autosolve) {
 				if ((n_lhs[n] == 1 && ((lhs[n][0].kind == CONSTANT && lhs[n][0].token.constant == 0.0)
@@ -166,10 +181,8 @@ char	*cp;
 #if	!LIBRARY
 			if (autocalc) {
 				/* the numerical input calculation */
-				for (i = 0; i < n_lhs[n]; i += 2) {
-					if (lhs[n][i].kind == VARIABLE && (lhs[n][i].token.variable & VAR_MASK) > SIGN) {
-						goto return_ok;		/* not numerical (contains a variable) */
-					}
+				if (!exp_is_numeric(lhs[n], n_lhs[n])) {
+					goto return_ok;		/* not numerical (contains a variable) */
 				}
 				/* make the expression an equation */
 				blt(rhs[n], lhs[n], n_lhs[n] * sizeof(token_type));
@@ -258,16 +271,20 @@ char	*cp;
 #if	!SECURE
 	/* handle shell escape */
 	if (*cp == '!') {
-		cp = skip_space(cp + 1);
-		if (*cp) {
-			return(!shell_out(cp));
-		}
 		cp1 = getenv("SHELL");
-		if (cp1) {
-			return(!shell_out(cp1));
+		if (cp1 == NULL) {
+			cp1 = "/bin/sh";
 		}
-		error(_("SHELL environment variable not set."));
-		return false;
+		if (access(cp1, X_OK)) {
+			error("Shell not found or not executable, check SHELL environment variable.");
+			return false;
+		}
+		cp = skip_space(cp + 1);
+		rv = shell_out(*cp ? cp : cp1);
+		if (rv) {
+			error(_("System command failed."));
+		}
+		return !rv;
 	}
 #endif
 	/* a quick way to get help */
@@ -279,7 +296,7 @@ char	*cp;
 /* See if the string pointed to by "cp" is a command. */
 /* If so, execute it. */
 	cp1 = cp;
-	while (*cp1 && isalpha(*cp1))
+	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if (((cp1 - cp) >= min(4, strlen(com_list[i].name)) && strncasecmp(cp, com_list[i].name, cp1 - cp) == 0)
@@ -313,9 +330,7 @@ char	*cp;
 					fp = fopen(filename, "w");
 				}
 				if (fp == NULL) {
-#if	!SILENT
-					printf(_("Can't open \"%s\" for writing.\n"), filename);
-#endif
+					error(_("Can't open redirected output file for writing."));
 					return false;
 				}
 				gfp = fp;
@@ -330,7 +345,7 @@ char	*cp;
 			/* execute the command */
 			rv = (*com_list[i].func)(buf2);
 #if	!SECURE
-			if (fp) {	/* if output redirected, close file */
+			if (fp) {	/* if output redirected, close output file */
 				if (gfp != stdout)
 					fclose(gfp);
 				else
@@ -349,7 +364,7 @@ char	*cp;
 
 #if	!SECURE
 /*
- * Execute a system command.
+ * Execute a system command.  Note that system(3) requires "/bin/sh".
  *
  * Returns exit value of command (0 if no error).
  */
@@ -361,6 +376,9 @@ char	*cp;
 
 	reset_attr();
 	rv = system(cp);
+	if (rv < 0) {
+		perror("system(3) call failed");
+	}
 	default_color();
 	return rv;
 }
@@ -393,7 +411,7 @@ char	*cp;	/* pointer to variable name string */
  */
 int
 display_command(i)
-int	i;	/* command index */
+int	i;	/* command table index of command */
 {
 	int	rows = 3;
 
@@ -425,6 +443,7 @@ char	*cp;
 	if (cp1 != cp) {
 		/* first, see if the argument matches any command names */
 		flag = false;
+next_argument:
 		for (i = 0; i < ARR_CNT(com_list); i++) {
 			if (strncasecmp(cp, com_list[i].name, cp1 - cp) == 0
 			    || (com_list[i].secondary_name && strncasecmp(cp, com_list[i].secondary_name, cp1 - cp) == 0)) {
@@ -432,8 +451,20 @@ char	*cp;
 				flag = true;
 			}
 		}
-		if (flag)
+		if (*cp == '!') {
+			P("! - Shell escape, execute shell and any specified system command.");
+			P("Usage: ![system-command]\n");
+			flag = true;
+		}
+		if (flag) {
+			cp1 = cp = skip_space(cp1);
+			while (*cp1 && !isspace(*cp1))
+				cp1++;
+			if (cp1 != cp) {
+				goto next_argument;
+			}
 			return true;
+		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
 			P("Command Usage Syntax");
 			P("--------------------");
@@ -469,29 +500,33 @@ char	*cp;
 			P("Right circular cone of radius \"r\" and height \"h\":");
 			P("    Volume = pi*r^2*h/3");
 			P("    Base surface area = pi*r^2    Side surface area = pi*r*(r^2 + h^2)^.5\n");
+
 			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
 			P("    Sum = (n - 2)*180 degrees     Sum = (n - 2)*pi radians");
 			return true;
 		}
 		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
-			P("Operators have precedence decreasing as indicated:\n");
+			P("Operators have precedence decreasing as indicated:");
 			P("    - negate");
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
 			P("    * multiply        / divide          % modulus");
-			P("    + add             - subtract\n");
+			P("    + add             - subtract");
+			P("    = equate");
 			P("Operators in the same precedence level are evaluated left to right.\n");
+
 			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Constants always start with a digit (0..9) or a period.");
-			P("Predefined constants and variables follow:\n");
+			P("Predefined constants and variables follow:");
 			P("    e or e# - the universal constant e (2.7182818284...).");
 			P("    pi or pi# - the universal constant pi (3.1415926535...).");
-			P("    i or i# - imaginary number (square root of -1).");
-			P("    sign, sign1, sign2, sign3, ... - may only be +1 or -1.");
-			P("    integer - may be any integer.");
-			P("    inf - infinity constant.\n");
-			P("Absolute value notation \"|x|\" and \"+/-\" are understood.");
+			P("    i or i# - the imaginary number (square root of -1).");
+			P("The above constants may also be used most anywhere variables are required.");
+			P("    sign, sign1, sign2, sign3, ... - variable that can only be +1 or -1.");
+			P("    integer - variable that may be any integer.");
+			P("    inf - infinity constant (not a variable).\n");
+
+			P("Absolute value notation \"|x|\" and dual polarity \"+/-\" are understood.");
 			return true;
 		}
 		if (is_all(cp)) {
@@ -510,11 +545,13 @@ char	*cp;
 	}
 	/* default help text: */
 	P("This help command is provided as a quick reference.");
+	P("Full documentation is available at the website \"www.mathomatic.org\".");
 	P("Type \"help all\" for a summary and syntax of the all commands.");
 	P("Type \"help usage\" for syntax of all commands.");
 	P("Type \"help expressions\" for help with entering expressions.");
 	P("Type \"help geometry\" for some commonly used geometric formulas.");
-	P("\"help\" or \"?\" followed by a command name will give info on a command.\n");
+	P("\"help\" or \"?\" followed by command names will give info on those commands.\n");
+
 	P("Available commands:");
 	for (i = 0; i < ARR_CNT(com_list); i++) {
 		if ((i % 5) == 0)
@@ -523,6 +560,7 @@ char	*cp;
 		for (; j > 0; j--)
 			fprintf(gfp, " ");
 	}
+
 	P("\n\nTo select an equation in memory, type the equation number at the main prompt.");
 	P("To solve the current equation, type the variable name at the main prompt.");
 	return true;
