@@ -3,7 +3,7 @@
  *
  * These are very low level, so they don't do much, polynomials only.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -12,10 +12,13 @@ static int	integrate_sub();
 static int	laplace_sub();
 static int	inv_laplace_sub();
 
+static int	constant_var_number = 1;	/* makes unique numbers for the constant of integration */
+
 /*
  * Make variable "v" always raised to a power,
  * unless it is on the right side of a power operator.
  */
+void
 make_powers(equation, np, v)
 token_type	*equation;	/* pointer to beginning of equation side */
 int		*np;		/* pointer to length of equation side */
@@ -195,15 +198,12 @@ long		v;		/* variable of integration */
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = 1.0;
-			j++;
 			blt(&equation[eloc+len+1], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 1;
-			k = eloc;
-			equation[k].level = mlevel;
-			equation[k].kind = OPERATOR;
-			equation[k].token.operatr = DIVIDE;
-			k++;
-			blt(&equation[k], &equation[i], len * sizeof(token_type));
+			equation[eloc].level = mlevel;
+			equation[eloc].kind = OPERATOR;
+			equation[eloc].token.operatr = DIVIDE;
+			blt(&equation[eloc+1], &equation[i], len * sizeof(token_type));
 			return true;
 		}
 	}
@@ -233,17 +233,21 @@ char	*cp;
 	long		v = 0;
 	token_type	*source, *dest;
 	int		n1, n2, *nps, *np;
-	int		def_flag;
+	int		definite_flag = false, constant_flag = false;
 	double		d1, integrate_order = 1.0;
+	char		temp_buf[50];
 
 	if (current_not_defined()) {
 		return false;
 	}
-	def_flag = (strcmp_tospace(cp, "definite") == 0);
-	if (def_flag) {
+	i = next_espace();
+	if (strcmp_tospace(cp, "definite") == 0) {
+		definite_flag = true;
+		cp = skip_param(cp);
+	} else if (strcmp_tospace(cp, "constant") == 0) {
+		constant_flag = true;
 		cp = skip_param(cp);
 	}
-	i = next_espace();
 	if (n_rhs[cur_equation]) {
 		source = rhs[cur_equation];
 		nps = &n_rhs[cur_equation];
@@ -256,9 +260,11 @@ char	*cp;
 		np = &n_lhs[i];
 	}
 	if (*cp) {
-		cp = parse_var2(&v, cp);
-		if (cp == NULL) {
-			return false;
+		if (isvarchar(*cp)) {
+			cp = parse_var2(&v, cp);
+			if (cp == NULL) {
+				return false;
+			}
 		}
 		if (*cp) {
 			integrate_order = strtod(cp, &cp);
@@ -267,7 +273,8 @@ char	*cp;
 			error(_("The order must be a positive integer."));
 			return false;
 		}
-	} else {
+	}
+	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
@@ -283,9 +290,32 @@ char	*cp;
 			error(_("Integration failed, not a polynomial."));
 			return false;
 		}
+		if (constant_flag) {
+			if (n1 + 2 > n_tokens) {
+				error_huge();
+			}
+			for (j = 0; j < n1; j++) {
+				dest[j].level++;
+			}
+			dest[n1].kind = OPERATOR;
+			dest[n1].level = 1;
+			dest[n1].token.operatr = PLUS;
+			n1++;
+			dest[n1].kind = VARIABLE;
+			dest[n1].level = 1;
+			snprintf(temp_buf, sizeof(temp_buf), "C_%d", constant_var_number);
+			if (parse_var(&dest[n1].token.variable, temp_buf) == NULL) {
+				return false;
+			}
+			n1++;
+			constant_var_number++;
+			if (constant_var_number < 0) {
+				constant_var_number = 1;
+			}
+		}
 		simp_loop(dest, &n1);
 	}
-	if (def_flag) {
+	if (definite_flag) {
 		my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 		if (!get_expr(tlhs, &n_tlhs)) {
 			return false;
@@ -314,7 +344,7 @@ char	*cp;
 		blt(&dest[n1], scratch, n2 * sizeof(token_type));
 		n1 += n2;
 	}
-	simpa_side(dest, &n1, false);
+	simpa_side(dest, &n1, false, false);
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
@@ -376,7 +406,6 @@ long		v;
 			equation[j].level = level;
 			equation[j].kind = CONSTANT;
 			equation[j].token.constant = -1.0;
-			j++;
 			blt(&equation[eloc+len+3], &equation[eloc], (*np - eloc) * sizeof(token_type));
 			*np += len + 3;
 			k = eloc;
@@ -393,7 +422,6 @@ long		v;
 			equation[k].level = mlevel + 1;
 			equation[k].kind = CONSTANT;
 			equation[k].token.constant = 1.0;
-			k++;
 			return true;
 		}
 	}
@@ -522,7 +550,7 @@ char	*cp;
 			return false;
 		}
 	}
-	if (extra_garbage(cp)) {
+	if (extra_characters(cp)) {
 		return false;
 	}
 	partial_flag = false;
@@ -606,6 +634,7 @@ char	*cp;
 		if (!prompt_var(&v))
 			return false;
 	}
+#if	!SILENT
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
@@ -617,8 +646,10 @@ char	*cp;
 		}
 	}
 	if (singularity) {
-		printf(_("Warning: Singularity detected, result of numerical integration may be wrong.\n"));
+		printf(_("Warning: Singularity detected for this expression, result will\n"));
+		printf(_("be wrong if the singularity is within the specified bounds.\n"));
 	}
+#endif
 	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
 	if (!get_expr(tlhs, &n_tlhs)) {
 		return false;
@@ -640,11 +671,13 @@ char	*cp;
 	if ((n_tlhs + n_trhs + 3) > n_tokens) {
 		error_huge();
 	}
+#if	!SILENT
 	if (trap_flag) {
-		debug_string(0, _("Approximating the definite integral using the trapezoid method..."));
+		printf(_("Approximating definite integral using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
-		debug_string(0, _("Approximating the definite integral using Simpson's rule..."));
+		printf(_("Approximating definite integral using Simpson's rule (%d partitions)...\n"), iterations);
 	}
+#endif
 	subst_constants(source, nps);
 	simp_loop(source, nps);
 	for (j = 0; j < n_trhs; j++) {
@@ -748,6 +781,7 @@ char	*cp;
 			n1 += 2;
 		}
 
+		/* simplify and approximate the partial result quickly: */
 		approximate_roots = true;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
@@ -755,9 +789,10 @@ char	*cp;
 		approximate_roots = false;
 
 		if (exp_contains_infinity(dest, n1)) {
-			error(_("Integration failed because result contains infinity or nan."));
+			error(_("Integration failed because result contains infinity or NaN (a singularity)."));
 			return false;
 		}
+		/* detect an ever growing result: */
 		switch (j) {
 		case 0:
 			break;
@@ -801,13 +836,16 @@ char	*cp;
 	for (; k < n1; k++)
 		dest[k].level++;
 
+	/* simplify and approximate the result as before: */
 	approximate_roots = true;
 	elim_loop(dest, &n1);
 	ufactor(dest, &n1);
 	simp_divide(dest, &n1);
 	approximate_roots = false;
 
-	debug_string(0, _("Integration successful."));
+#if	!SILENT
+	printf(_("Integration successful.\n"));
+#endif
 	if (n_rhs[cur_equation]) {
 		blt(lhs[i], lhs[cur_equation], n_lhs[cur_equation] * sizeof(token_type));
 		n_lhs[i] = n_lhs[cur_equation];
