@@ -2,12 +2,13 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
 #define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
+#define MODULUS_STRING	" % "	/* string displayed for the modulus binary operator */
 
 #define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } len += strlen(str); }
 
@@ -43,6 +44,7 @@ int	cur_pos;	/* current position in the current line on the screen */
 /*
  * Turn color off if color mode is on.
  */
+void
 reset_attr()
 {
 	cur_color = -1;
@@ -60,6 +62,7 @@ reset_attr()
  * Set the current color on the display.
  * Range is 0 to INT_MAX.
  */
+void
 set_color(color)
 int	color;
 {
@@ -87,6 +90,7 @@ int	color;
 /*
  * Set the normal text color.
  */
+void
 default_color()
 {
 	set_color(0);
@@ -96,7 +100,7 @@ default_color()
  * Display the expression or equation stored in equation space "n" in single-line format.
  * The equation space is not modified.
  *
- * Return true if successful.
+ * Return length (number of screen columns) of output line.
  */
 int
 list1_sub(n, export_flag)
@@ -104,18 +108,20 @@ int	n;		/* equation space number */
 int	export_flag;	/* true for exportable format (readable by other math programs) */
 			/* 1 for Maxima, 2 for other */
 {
+	int	len = 0;
+
 	if (n_lhs[n] <= 0)
-		return false;
+		return len;
 	if (!export_flag && !high_prec) {
-		fprintf(gfp, "#%d: ", n + 1);
+		len += fprintf(gfp, "#%d: ", n + 1);
 	}
-	list_proc(lhs[n], n_lhs[n], export_flag);
+	len += list_proc(lhs[n], n_lhs[n], export_flag);
 	if (n_rhs[n]) {
-		fprintf(gfp, EQUATE_STRING);
-		list_proc(rhs[n], n_rhs[n], export_flag);
+		len += fprintf(gfp, EQUATE_STRING);
+		len += list_proc(rhs[n], n_rhs[n], export_flag);
 	}
 	if (export_flag == 1) {
-		fprintf(gfp, ";");
+		len += fprintf(gfp, ";");
 	}
 	if (high_prec) {
 #if	CYGWIN
@@ -126,20 +132,21 @@ int	export_flag;	/* true for exportable format (readable by other math programs)
 	} else {
 		fprintf(gfp, "\n\n");
 	}
-	return true;
+	return len;
 }
 
 /*
  * Display the expression or equation stored in equation space "n".
  *
- * Return true if successful.
+ * Return the total width of the output (number of screen columns)
+ * or zero on failure.
  */
 int
 list_sub(n)
 int	n;	/* equation space number */
 {
 	if (n_lhs[n] <= 0)
-		return false;
+		return 0;
 	if (display2d) {
 		/* display in fraction format */
 		make_fractions_and_group(n);
@@ -157,6 +164,7 @@ int	n;	/* equation space number */
 }
 
 #if	!SILENT
+void
 list_debug(level, p1, n1, p2, n2)
 int		level;
 token_type	*p1;
@@ -177,16 +185,16 @@ int		n2;
 #endif
 
 /*
- * Convert variable "v" to an ASCII variable name.
- * The ASCII variable name is stored in global "var_str[]".
+ * Convert the passed Mathomatic variable to an ASCII variable name.
+ * The ASCII variable name is stored in the global var_str[].
  *
  * Return length of variable name (number of ASCII characters).
  *
- * If "lang_code == 0", use standard Mathomatic format.
- * If "lang_code == 1", make variable name C language compatible.
- * If "lang_code == 2", assume Java.
- * If "lang_code == 3", assume Python.
- * If "lang_code" is negative, create an exportable variable name: -1 for Maxima, -2 for other.
+ * If (lang_code == 0), use standard Mathomatic format.
+ * If (lang_code == 1), make variable name C language compatible.
+ * If (lang_code == 2), make variable name Java compatible.
+ * If (lang_code == 3), make variable name Python compatible.
+ * If lang_code is negative, create an exportable variable name: -1 for Maxima, -2 for other.
  */
 int
 list_var(v, lang_code)
@@ -280,7 +288,7 @@ int	lang_code;	/* language code */
 	if (cp) {
 		my_strlcpy(var_str, cp, sizeof(var_str));
 	} else {
-		my_strlcpy(var_str, _("bad_variable"), sizeof(var_str));
+		my_strlcpy(var_str, "bad_variable", sizeof(var_str));
 	}
 	j = (v >> VAR_SHIFT) & SUBSCRIPT_MASK;
 	if (j) {
@@ -337,7 +345,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 					len += fprintf(gfp, "%.20g", equation[i].token.constant);
 				}
 			} else if (finance_option) {
-				len += fprintf(gfp, "%.2f", equation[i].token.constant);
+				len += fprintf(gfp, "%.*f", finance_option, equation[i].token.constant);
 			} else {
 				len += fprintf(gfp, "%.*g", precision, equation[i].token.constant);
 			}
@@ -362,7 +370,7 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				cp = "/";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				cp = "^";
@@ -406,7 +414,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
 	len += 2;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
-		error(_("Can't malloc()."));
+		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
 	list_string_sub(lhs[n], n_lhs[n], cp, export_flag);
@@ -439,7 +447,7 @@ int		export_flag;
 	len++;
 	cp = (char *) malloc(len);
 	if (cp == NULL) {
-		error(_("Can't malloc()."));
+		error(_("Out of memory (can't malloc(3))."));
 		return NULL;
 	}
 	list_string_sub(equation, n, cp, export_flag);
@@ -494,7 +502,7 @@ int		export_flag;
 					snprintf(buf, sizeof(buf), "%.20g", equation[i].token.constant);
 				}
 			} else if (finance_option) {
-				snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, equation[i].token.constant);
 			} else {
 				snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
@@ -520,7 +528,7 @@ int		export_flag;
 				cp = "/";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				cp = "^";
@@ -563,6 +571,7 @@ int		n;		/* length of expression */
 /*
  * Display an equation as C, Java, or Python code.
  */
+void
 list_c_equation(en, language, int_flag)
 int	en;		/* equation space number */
 int	language;
@@ -688,11 +697,11 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
 				cp = " / ";
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
 				break;
 			case POWER:
 				if (int_flag || language > 1) {
-					cp = " ** ";
+					cp = "**";
 				} else {
 					cp = ", ";
 				}
@@ -716,7 +725,8 @@ int		int_flag;	/* integer arithmetic flag, should work with any language */
  * Display an equation space in multi-line fraction format.
  * Use color if available.
  *
- * Return true if successful.
+ * Return the total width of the output (that is, the required number of screen columns)
+ * or zero on failure.
  */
 int
 flist_sub(n)
@@ -724,14 +734,14 @@ int	n;	/* equation space number */
 {
 	int	sind;
 	char	buf[30];
-	int	len, len2, len3;
+	int	len, len2, len3, width;
 	int	pos;
 	int	high, low;
 	int	max_line, min_line;
 	int	max2_line, min2_line;
 
 	if (n_lhs[n] <= 0)
-		return false;
+		return 0;
 	len = snprintf(buf, sizeof(buf), "#%d: ", n + 1);
 	cur_line = 0;
 	cur_pos = 0;
@@ -763,7 +773,9 @@ make_smaller:
 		len2 = 0;
 		len3 = 0;
 	}
-	if (screen_columns && gfp == stdout && max(len + len2, len3) >= screen_columns) {
+	width = max(len + len2, len3);
+	if (screen_columns && gfp == stdout && width >= screen_columns) {
+		/* output too wide to fit screen, output in single-line format */
 		return list1_sub(n, false);
 	}
 	fprintf(gfp, "\n");
@@ -792,7 +804,7 @@ make_smaller:
 		}
 	}
 	fprintf(gfp, "\n");
-	return true;
+	return width;
 }
 
 /*
@@ -956,7 +968,7 @@ check_again:
 				equation[i].token.constant = 0.0; /* fix -0 */
 			}
 			if (finance_option) {
-				len += snprintf(buf, sizeof(buf), "%.2f", equation[i].token.constant);
+				len += snprintf(buf, sizeof(buf), "%.*f", finance_option, equation[i].token.constant);
 			} else {
 				len += snprintf(buf, sizeof(buf), "%.*g", precision, equation[i].token.constant);
 			}
@@ -969,34 +981,52 @@ check_again:
 				fprintf(gfp, "%s", var_str);
 			break;
 		case OPERATOR:
-			cp = _("(unknown operator)");
+			/* len must be incremented here by the number of columns the display of the operator takes */
 			switch (equation[i].token.operatr) {
 			case PLUS:
 				cp = " + ";
+				len += 3;
 				break;
 			case MINUS:
-				cp = " - ";
+				if (html_flag) {
+					cp = " &minus; ";
+				} else {
+					cp = " - ";
+				}
+				len += 3;
 				break;
 			case TIMES:
-				cp = "*";
+				if (html_flag) {
+					cp = "&middot;";
+				} else {
+					cp = "*";
+				}
+				len++;
 				break;
 			case DIVIDE:
 				cp = "/";
+				len++;
 				break;
 			case MODULUS:
-				cp = " % ";
+				cp = MODULUS_STRING;
+				len += strlen(cp);
 				break;
 			case POWER:
 				cp = "^";
+				len++;
 				break;
 			case FACTORIAL:
 				cp = "!";
+				len++;
 				i++;
 				break;
+			default:
+				cp = _("(unknown operator)");
+				len += strlen(cp);
+				break;
 			}
 			if (oflag)
 				fprintf(gfp, "%s", cp);
-			len += strlen(cp);
 			break;
 		}
 	}
