@@ -1,7 +1,7 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -10,20 +10,151 @@
 
 int	point_flag;		/* point to location of parse error if true */
 
+/*
+ * Convert all alphabetic characters in a string to lower case.
+ */
+void
+str_tolower(cp)
+char	*cp;
+{
+	for (; *cp; cp++) {
+		if (isupper(*cp))
+			*cp = tolower(*cp);
+	}
+}
+
+/*
+ * Display an up arrow pointing to the error under the input string.
+ */
+void
+put_up_arrow(cnt, cp)
+int	cnt;	/* position of error, relative to "input_column" */
+char	*cp;	/* error message */
+{
+#if	!SILENT
+	int	 i;
+
+	if (isatty(0) && point_flag) {
+		for (i = 0; i < (input_column + cnt); i++) {
+			printf(" ");
+		}
+		printf("^ ");
+	}
+#endif
+	error(cp);
+}
+
 /*
  * Return true if character is a valid starting variable character.
  */
-static inline int
+int
 isvarchar(ch)
 int	ch;
 {
 	return(ch == '_' || (special_variable_character && ch == special_variable_character) || isalpha(ch));
 }
 
+/*
+ * Parenthesize an operator.
+ */
+void
+binary_parenthesize(p1, n, i)
+token_type	*p1;	/* pointer to expression */
+int		n;	/* length of expression */
+int		i;	/* location of operator to parenthesize in expression */
+{
+	int	j;
+	int	level;
+
+	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
+		error("Internal error in arguments to binary_parenthesize().");
+		longjmp(jmp_save, 13);
+	}
+	level = p1[i].level++;
+	if (p1[i-1].level++ > level) {
+		for (j = i - 2; j >= 0; j--) {
+			if (p1[j].level <= level)
+				break;
+			p1[j].level++;
+		}
+	}
+	if (p1[i+1].level++ > level) {
+		for (j = i + 2; j < n; j++) {
+			if (p1[j].level <= level)
+				break;
+			p1[j].level++;
+		}
+	}
+}
+
+/*
+ * Handle and remove the special NEGATE operator.
+ */
+void
+handle_negate(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	i;
+
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr == NEGATE) {
+			equation[i].token.operatr = TIMES;
+			binary_parenthesize(equation, *np, i);
+		}
+	}
+}
+
+/*
+ * Parenthesize most operators so that expression evaluation is in the correct order.
+ * Gives different operators on the same level in an expression the correct priority.
+ *
+ * organize() should be called after this to remove unneeded parentheses.
+ */
+void
+give_priority(equation, np)
+token_type	*equation;	/* pointer to expression */
+int		*np;		/* pointer to expression length */
+{
+	int	i;
+
+/* Higher priority (precedence) operators need to be parenthesized first: */
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr == FACTORIAL) {
+			binary_parenthesize(equation, *np, i);
+		}
+	}
+
+	if (right_associative_power) {
+		for (i = *np - 2; i >= 1; i -= 2) {	/* count down (right to left) */
+			if (equation[i].token.operatr == POWER) {
+				binary_parenthesize(equation, *np, i);
+			}
+		}
+	} else {
+		for (i = 1; i < *np; i += 2) {		/* count up (left to right) */
+			if (equation[i].token.operatr == POWER) {
+				binary_parenthesize(equation, *np, i);
+			}
+		}
+	}
+
+	for (i = 1; i < *np; i += 2) {
+		switch (equation[i].token.operatr) {
+		case TIMES:
+		case DIVIDE:
+		case MODULUS:
+			binary_parenthesize(equation, *np, i);
+			break;
+		}
+	}
+}
+
 /*
  * Parse an equation string into equation space "n".
  *
  * Returns the new string position or NULL on error.
+ * Currently, there can be no more to parse in the string when this returns.
  */
 char	*
 parse_equation(n, cp)
@@ -47,9 +178,10 @@ char	*cp;	/* pointer to the beginning of the equation character string */
  * This is a simple, non-recursive mathematical expression parser.
  * To parse, the character string is sequentially read and stored
  * in the internal storage format.
- * Any syntax errors are carefully reported with a NULL return.
+ * The maximum length that can be parsed is the size of an equation side.
+ * Any syntax and other errors are carefully reported and give a NULL return.
  *
- * Returns the new string position or NULL on error.
+ * Returns the new string position or NULL if error.
  */
 char	*
 parse_section(equation, np, cp)
@@ -96,9 +228,9 @@ char		*cp;		/* string to parse */
 			continue;
 		case '=':
 		case ';':
-		case 0:
+		case '\0':
 		case '\n':
-			goto p_out;
+			goto p_out;	/* terminator encountered */
 		}
 		if (n > (n_tokens - 6)) {
 			error_huge();
@@ -111,7 +243,7 @@ char		*cp;		/* string to parse */
 					error(_("Too many nested absolute values."));
 					return(NULL);
 				}
-				cur_level += 2;
+				cur_level += 3;
 				abs_array[abs_count++] = cur_level;
 			} else {
 				if (abs_count <= 0 || cur_level != abs_array[--abs_count]) {
@@ -126,6 +258,7 @@ char		*cp;		/* string to parse */
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = 2.0;
 				n++;
+				cur_level--;
 				equation[n].level = cur_level;
 				equation[n].kind = OPERATOR;
 				equation[n].token.operatr = POWER;
@@ -143,7 +276,7 @@ char		*cp;		/* string to parse */
 				goto syntax_error;
 			}
 			if (cp[1] == '!') {
-				put_up_arrow((int) (cp - cp_start), _("Multifactorial not supported yet."));
+				put_up_arrow((int) (cp - cp_start), _("Multifactorial not supported, use (x!)! for the factorial of x factorial."));
 				return(NULL);
 			}
 			equation[n].level = cur_level;
@@ -280,10 +413,10 @@ parse_power:
 				equation[n].token.operatr = TIMES;
 				n++;
 			}
-			if (strncmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
+			if (strncasecmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
 			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
-				equation[n].token.constant = HUGE_VAL;	/* the infinity constant */
+				equation[n].token.constant = INFINITY;	/* the infinity constant */
 				cp += strlen(INFINITY_NAME);
 			} else {
 				equation[n].kind = VARIABLE;
@@ -312,7 +445,7 @@ p_out:
 	*np = n;
 	if (n) {
 		handle_negate(equation, np);
-		prior_sub(equation, np);
+		give_priority(equation, np);
 		organize(equation, np);
 	}
 	input_column += (cp - cp_start);
@@ -365,7 +498,7 @@ char	*cp;
 		buf[i++] = *cp1++;
 	}
 	buf[i] = '\0';
-	if ((*strcmpfunc)(buf, INFINITY_NAME) == 0) {
+	if (strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Invalid variable."));
 		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
@@ -394,20 +527,18 @@ char	*cp;
 			buf[i++] = *cp1++;
 		}
 		buf[i] = '\0';
+		if ((*strcmpfunc)(buf, "i") == 0) {
+			*vp = IMAGINARY;
+			return(cp1);
+		}
+		if ((*strcmpfunc)(buf, "e") == 0) {
+			*vp = V_E;
+			return(cp1);
+		}
 		if ((*strcmpfunc)(buf, "pi") == 0) {
 			*vp = V_PI;
 			return(cp1);
 		}
-		if (e_i_as_constants) {
-			if ((*strcmpfunc)(buf, "i") == 0) {
-				*vp = IMAGINARY;
-				return(cp1);
-			}
-			if ((*strcmpfunc)(buf, "e") == 0) {
-				*vp = V_E;
-				return(cp1);
-			}
-		}
 		vtmp = 0;
 		for (i = 0; var_names[i]; i++) {
 			if ((*strcmpfunc)(buf, var_names[i]) == 0) {
@@ -452,18 +583,20 @@ char	*cp;
 }
 
 /*
- * Set point_flag if pointing to the input error works for the passed string.
+ * This should be called for all line input.
+ * Set point_flag to true if pointing to the input error will work for the passed string.
  * Truncate string to the actual content.
  */
+void
 set_error_level(cp)
 char	*cp;	/* input string */
 {
 	char	*cp1;
 
 	point_flag = true;
-/* handle comments */
+/* handle comments and the DOS EOF character (control-Z) by truncating the string where found */
 	for (cp1 = cp; *cp1; cp1++) {
-		if (*cp1 == ';') {
+		if (*cp1 == ';' || *cp1 == '\032') {
 			*cp1 = '\0';
 			break;
 		}
@@ -487,27 +620,7 @@ char	*cp;	/* input string */
 }
 
 /*
- * Display an up arrow pointing to the error under the input string.
- */
-put_up_arrow(cnt, cp)
-int	cnt;	/* position of error, relative to "input_column" */
-char	*cp;	/* error message */
-{
-#if	!SILENT
-	int	 i;
-
-	if (isatty(0) && point_flag) {
-		for (i = 0; i < (input_column + cnt); i++) {
-			printf(" ");
-		}
-		printf("^ ");
-	}
-#endif
-	error(cp);
-}
-
-/*
- * Return true if passed variable is a constant.
+ * Return true if passed variable "v" is a constant.
  * Return value of constant in "*dp".
  */
 int
@@ -550,96 +663,6 @@ int		*np;
 	return modified;
 }
 
-/*
- * Parenthesize an operator.
- */
-binary_parenthesize(equation, n, i)
-token_type	*equation;	/* pointer to expression */
-int		n;		/* length of expression */
-int		i;		/* location of operator to parenthesize in expression */
-{
-	int	j;
-	int	level;
-
-	level = equation[i].level++;
-	if (equation[i-1].level++ > level) {
-		for (j = i - 2; j >= 0; j--) {
-			if (equation[j].level <= level)
-				break;
-			equation[j].level++;
-		}
-	}
-	if (equation[i+1].level++ > level) {
-		for (j = i + 2; j < n; j++) {
-			if (equation[j].level <= level)
-				break;
-			equation[j].level++;
-		}
-	}
-}
-
-/*
- * Handle and remove the special NEGATE operator.
- */
-handle_negate(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	i;
-
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == NEGATE) {
-			equation[i].token.operatr = TIMES;
-			binary_parenthesize(equation, *np, i);
-		}
-	}
-}
-
-/*
- * Give different operators on the same level
- * in an expression the correct priority.
- *
- * organize() should be called after this.
- */
-prior_sub(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	i;
-
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == FACTORIAL) {
-			binary_parenthesize(equation, *np, i);
-		}
-	}
-	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == POWER) {
-			binary_parenthesize(equation, *np, i);
-		}
-	}
-	for (i = 1; i < *np; i += 2) {
-		switch (equation[i].token.operatr) {
-		case TIMES:
-		case DIVIDE:
-		case MODULUS:
-			binary_parenthesize(equation, *np, i);
-			break;
-		}
-	}
-}
-
-/*
- * Convert all alphabetic characters in a string to lower case.
- */
-str_tolower(cp)
-char	*cp;
-{
-	for (; *cp; cp++) {
-		if (isupper(*cp))
-			*cp = tolower(*cp);
-	}
-}
-
 #ifndef	my_strlcpy
 /*
  * A very efficient strlcpy().
