@@ -1,11 +1,12 @@
 /*
  * Mathomatic simplifying and polynomial routines.
  * Includes polynomial and smart division, polynomial factoring, etc.
+ * tlhs and trhs are wiped out by most of these routines.
  *
  * The polynomial division and GCD routines here are not recursive, due to static
  * storage areas.  This limits the polynomial GCD to univariate operation.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -14,9 +15,9 @@
 
 /* These are a non-standard size and must only be used for temp storage: */
 token_type	divisor[DIVISOR_SIZE];		/* static data areas for polynomial and smart division */
-int		n_divisor;
+int		n_divisor;			/* length of expression in divisor[] */
 token_type	quotient[DIVISOR_SIZE];
-int		n_quotient;
+int		n_quotient;			/* length of expression in quotient[] */
 token_type	gcd_divisor[DIVISOR_SIZE];	/* static data area for polynomial GCD routine */
 int		len_d;				/* length of expression in gcd_divisor[] */
 
@@ -41,29 +42,30 @@ sort_type	*p1, *p2;
 
 /*
  * Return true if passed expression is strictly a single polynomial term in variable v.
+ * The exponent may be anything, as long as it doesn't contain the variable.
  */
 static inline int
-poly_in_v_sub(equation, n, v, allow_divides)
-token_type	*equation;	/* expression pointer */
+poly_in_v_sub(p1, n, v, allow_divides)
+token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
 long		v;		/* variable */
-int		allow_divides;	/* allow division by variable */
+int		allow_divides;	/* if true, allow division by variable */
 {
 	int	i, k;
 	int	level, vlevel;
 	int	count;
 
-	level = min_level(equation, n);
+	level = min_level(p1, n);
 	for (i = 0, count = 0; i < n; i += 2) {
-		if (equation[i].kind == VARIABLE && equation[i].token.variable == v) {
+		if (p1[i].kind == VARIABLE && p1[i].token.variable == v) {
 			count++;
 			if (count > 1)
 				return false;
-			vlevel = equation[i].level;
+			vlevel = p1[i].level;
 			if (vlevel == level || vlevel == (level + 1)) {
 				for (k = 1; k < n; k += 2) {
-					if (equation[k].level == level) {
-						switch (equation[k].token.operatr) {
+					if (p1[k].level == level) {
+						switch (p1[k].token.operatr) {
 						case DIVIDE:
 							if (!allow_divides && k == (i - 1))
 								return false;
@@ -78,8 +80,8 @@ int		allow_divides;	/* allow division by variable */
 					}
 				}
 				if (vlevel == (level + 1)) {
-					if ((i + 1) < n && equation[i+1].level == vlevel
-					    && equation[i+1].token.operatr == POWER) {
+					if ((i + 1) < n && p1[i+1].level == vlevel
+					    && p1[i+1].token.operatr == POWER) {
 						continue;
 					}
 				} else {
@@ -97,18 +99,18 @@ int		allow_divides;	/* allow division by variable */
  * The exponents may be anything, as long as they don't contain the variable.
  */
 int
-poly_in_v(equation, n, v, allow_divides)
-token_type	*equation;	/* expression pointer */
+poly_in_v(p1, n, v, allow_divides)
+token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
 long		v;		/* variable */
-int		allow_divides;	/* allow variable to be right of a divide (negative exponents) */
+int		allow_divides;	/* allow variable to be right of a divide (negative exponents) as a polynomial term */
 {
 	int	i, j;
 
 	for (i = 1, j = 0;; i += 2) {
-		if (i >= n || (equation[i].level == 1
-		    && (equation[i].token.operatr == PLUS || equation[i].token.operatr == MINUS))) {
-			if (!poly_in_v_sub(&equation[j], i - j, v, allow_divides)) {
+		if (i >= n || (p1[i].level == 1
+		    && (p1[i].token.operatr == PLUS || p1[i].token.operatr == MINUS))) {
+			if (!poly_in_v_sub(&p1[j], i - j, v, allow_divides)) {
 				return false;
 			}
 			j = i + 1;
@@ -207,16 +209,22 @@ int		*np, loc, len, level;
 		blt(jmp_save, save_save, sizeof(jmp_save));
 		return(modified || symbolic_modified);
 	}
-/* First factor polynomials with repeated factors: */
+/* First factor polynomials with repeated factors */
+/* using poly_gcd(polynomial, v * differentiate(polynomial, v)) to discover the factors: */
 	for (count = 1;; count++) {
 		blt(trhs, &equation[loc1], len * sizeof(token_type));
 		n_trhs = len;
 		uf_simp(trhs, &n_trhs);
-		if (level1_plus_count(trhs, n_trhs) < 2)
+		if (level1_plus_count(trhs, n_trhs) < 2) {
+/* must be at least 2 level 1 additive operators to be factorable */
 			goto skip_factor;
+		}
+/* create a variable list with counts of the number of times each variable occurs: */
 		last_v = 0;
 		for (vc = 0; vc < ARR_CNT(va); vc++) {
-			for (i = 0, cnt = 0, v1 = -1; i < n_trhs; i += 2) {
+			cnt = 0;
+			v1 = -1;
+			for (i = 0; i < n_trhs; i += 2) {
 				if (trhs[i].kind == VARIABLE && trhs[i].token.variable > last_v) {
 					if (v1 == -1 || trhs[i].token.variable < v1) {
 						v1 = trhs[i].token.variable;
@@ -233,6 +241,7 @@ int		*np, loc, len, level;
 			va[vc].count = cnt;
 		}
 		side_debug(3, &equation[loc1], len);
+/* find a valid polynomial base variable "v": */
 		cnt = -1;
 		if (v) {
 			if (!poly_in_v(trhs, n_trhs, v, true)) {
@@ -527,10 +536,10 @@ long		v;
 {
 	int		count;
 
+	if (llen > n_tokens || slen > min(ARR_CNT(gcd_divisor), n_tokens))
+		return 0;
 	blt(trhs, larger, llen * sizeof(token_type));
 	n_trhs = llen;
-	if (slen > min(ARR_CNT(gcd_divisor), n_tokens))
-		return 0;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
 	if (!remove_factors())
@@ -588,10 +597,10 @@ long		v;
 	debug_string(3, "Entering poly2_gcd():");
 	side_debug(3, larger, llen);
 	side_debug(3, smaller, slen);
+	if (llen > n_tokens || slen > min(ARR_CNT(gcd_divisor), n_tokens))
+		return 0;
 	blt(trhs, larger, llen * sizeof(token_type));
 	n_trhs = llen;
-	if (slen > min(ARR_CNT(gcd_divisor), n_tokens))
-		return 0;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
 	blt(save_save, jmp_save, sizeof(jmp_save));
@@ -646,16 +655,16 @@ long		v;
  * the expression must be integer if this returns true.
  */
 int
-is_integer_expr(equation, n)
-token_type	*equation;	/* expression pointer */
-int		n;		/* length of expression */
+is_integer_expr(p1, n)
+token_type	*p1;	/* expression pointer */
+int		n;	/* length of expression */
 {
 	int	i;
 
 	for (i = 0; i < n; i++) {
-		if ((equation[i].kind == OPERATOR && equation[i].token.operatr == DIVIDE)
-		    || (equation[i].kind == CONSTANT && fmod(equation[i].token.constant, 1.0))
-		    || (equation[i].kind == VARIABLE && equation[i].token.variable <= IMAGINARY)) {
+		if ((p1[i].kind == OPERATOR && p1[i].token.operatr == DIVIDE)
+		    || (p1[i].kind == CONSTANT && fmod(p1[i].token.constant, 1.0))
+		    || (p1[i].kind == VARIABLE && p1[i].token.variable <= IMAGINARY)) {
 			return false;
 		}
 	}
@@ -663,15 +672,15 @@ int		n;		/* length of expression */
 }
 
 /*
- * This routine is a modulus simplifier for equation sides.
+ * This routine is a modulus (%) simplifier for equation sides.
  * It does polynomial division, so tlhs and trhs are wiped out.
  *
  * Return true if expression was modified.
  */
 int
 mod_simp(equation, np)
-token_type	*equation;
-int		*np;
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
 {
 	return mod_recurse(equation, np, 0, 1);
 }
@@ -1392,7 +1401,7 @@ int		len2;		/* length of divisor */
 			}
 			flag = false;
 			j = i + 1;
-		} else if (tlhs[i].kind == VARIABLE && (tlhs[i].token.variable & VAR_MASK) > SIGN) {
+		} else if (tlhs[i].kind == VARIABLE && tlhs[i].token.variable != IMAGINARY) {
 			flag = true;
 		}
 		if (i >= n_tlhs)
@@ -1667,7 +1676,9 @@ long		*vp1;		/* variable pointer */
 
 	last_v = 0;
 	for (vc = 0; vc < ARR_CNT(va); vc++) {
-		for (i = 0, cnt = 0, v1 = -1; i < n1; i += 2) {
+		cnt = 0;
+		v1 = -1;
+		for (i = 0; i < n1; i += 2) {
 			if (p1[i].kind == VARIABLE && p1[i].token.variable > last_v) {
 				if (v1 == -1 || p1[i].token.variable < v1) {
 					v1 = p1[i].token.variable;
@@ -1709,6 +1720,7 @@ long		*vp1;		/* variable pointer */
 
 #define	VALUE_CNT	3
 
+void
 term_value(dp, p1, n1, loc)
 double		*dp;
 token_type	*p1;
