@@ -6,8 +6,12 @@
 
 #include <stdio.h>
 
+#define	true	1
+#define	false	0
+
+#if	false
 /*
- * Return the truncated integer square root of "y" using longs.
+ * Return the truncated integer square root using the Babylonian iterative approximation method.
  * Return -1 on error.
  */
 long
@@ -20,7 +24,7 @@ lsqrt(long y)
 
 	if (y <= 0) {
 		if (y != 0) {
-			fprintf(stderr, "Domain error in lsqrt().\n");
+			fprintf(stderr, "Domain error in %s().\n", __func__);
 			return -1L;
 		}
 		return 0L;
@@ -43,8 +47,42 @@ lsqrt(long y)
 	}
 /* optional test code to make sure that the answer is right: */
 	if ((long long) x_old * x_old > y || ((long long) x_old + 1) * ((long long) x_old + 1) <= y) {
-		fprintf(stderr, "Error in lsqrt().\n");
+		fprintf(stderr, "Error in %s().\n", __func__);
 		return -1L;
 	}
 	return x_old;
 }
+#else
+/*
+ * Faster and simpler binary digit algorithm for calculating the truncated integer square root.
+ * Returns -1 on error.
+ */
+long
+lsqrt(long num)
+{
+	long op = num;
+	long res = 0;
+	long one = 1L << (sizeof(num) * 8 - 2); // The second-to-top bit is set
+
+	if (num <= 0) {
+		if (num != 0) {
+			fprintf(stderr, "Domain error in %s().\n", __func__);
+			return -1L;
+		}
+		return 0L;
+	}
+// "one" starts at the highest power of four <= the argument.
+	while (one > op)
+		one >>= 2;
+
+	while (one != 0) {
+		if (op >= res + one) {
+			op -= res + one;
+			res += 2 * one;
+		}
+		res >>= 1;
+		one >>= 2;
+	}
+	return res;
+}
+#endif
