@@ -1,7 +1,7 @@
 /*
- * Mathomatic solve routines.
+ * Mathomatic symbolic solve routines.
  *
- * Copyright (C) 1987-2006 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
@@ -193,7 +193,7 @@ no_simp:
 		simps_side(trhs, &n_trhs, zsolve);
 	}
 left_again:
-	worked = 1;
+	worked = true;
 	uf_flag = false;
 see_work:
 	/* See if we have solved the equation. */
@@ -348,8 +348,7 @@ zero_simp:
 			if (p1 >= ep) {
 				if (found_count == 0) { /* if solve variable no longer in LHS */
 					for (i = 0; i < n_trhs; i += 2) {
-						if (trhs[i].kind == VARIABLE
-						    && trhs[i].token.variable == v) {
+						if (trhs[i].kind == VARIABLE && trhs[i].token.variable == v) {
 							/* solve variable in RHS */
 							return false;
 						}
@@ -400,7 +399,7 @@ zero_simp:
 					goto left_again;
 				}
 				if (worked && !uf_flag) {
-					worked--;
+					worked = false;
 					debug_string(1, _("Unfactoring..."));
 					partial_flag = false;
 					uf_simp(tlhs, &n_tlhs);
@@ -487,14 +486,15 @@ zero_simp:
 						    || fabs(numerator) != 1.0 || denominator < 2.0) {
 							continue;
 						}
-						for (j = i - 1; (j >= 0) && tlhs[j].level >= tlhs[i].level; j--) {
+						for (j = i - 1; j >= 0 && tlhs[j].level >= tlhs[i].level; j--) {
 							if (tlhs[j].kind == VARIABLE && tlhs[j].token.variable == v) {
 								if (b1) {
 									if (fabs(b1->token.constant) < fabs(tlhs[i+1].token.constant)) {
 										b1 = &tlhs[i+1];
 									}
-								} else
+								} else {
 									b1 = &tlhs[i+1];
+								}
 								break;
 							}
 						}
@@ -528,9 +528,8 @@ zero_simp:
 					return false;
 				if (zero_solved) {
 					qtries++;
-				} else {
-					zero_solved = true;
 				}
+				zero_solved = true;
 				if (quad_solve(v)) {
 					goto left_again;
 				} else {
@@ -677,13 +676,13 @@ end:
 
 /*
  * Quadratic and biquadratic solve routine.
- * Solves any equation of the form "0 = ax^(2n)+bx^n+c" for "x",
- * where "x" is an expression containing passed variable "v",
- * and "n" is a constant.
+ * Solves any equation of the form "0 = ax^(2n)+bx^n+c" for "x^n",
+ * where "x" is an expression containing the solve variable,
+ * and "n" is a constant.  Uses the quadratic formula.
  *
- * The equation to solve is in tlhs and trhs, it must be solved for zero.
+ * The equation to solve is in tlhs and trhs, it must already be solved for zero.
  *
- * Return true if successful.
+ * Return true if successful, with solved equation in tlhs and trhs.
  */
 static int
 quad_solve(v)
@@ -781,6 +780,10 @@ long	v;	/* solve variable */
 	}
 	if (high_power == 0.0)
 		return false;
+#if	!SILENT
+	list_var(v, 0);
+	printf(_("Equation is a degree %.*g polynomial in %s.\n"), precision, high_power, var_str);
+#endif
 	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
 		opx1 = MINUS;
 	else
@@ -995,7 +998,7 @@ big_bbreak:
 	} else if (high_power == 4.0) {
 		printf(_("Equation was biquadratic.\n"));
 	} else {
-		printf(_("Equation was a degree %.*g quadratic.\n"), precision, high_power);
+		printf(_("Equation was solved with the quadratic formula.\n"));
 	}
 #endif
 	return true;
