@@ -1,20 +1,116 @@
 /*
  * Group and combine denominators.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
 static int	sf_recurse();
 static int	sf_sub();
-static void	group_recurse();
+
+static void
+group_recurse(equation, np, loc, level)
+token_type	*equation;
+int		*np, loc, level;
+{
+	int		i;
+	int		len;
+	int		di, edi;
+	int		grouper = false;
+	int		e1;
+
+	for (i = loc; i < *np && equation[i].level >= level;) {
+		if (equation[i].level > level) {
+			group_recurse(equation, np, i, level + 1);
+			i++;
+			for (; i < *np && equation[i].level > level; i += 2)
+				;
+			continue;
+		}
+		i++;
+	}
+	e1 = i;
+	di = -1;
+	edi = e1;
+	for (i = loc + 1; i < e1; i += 2) {
+		if (equation[i].level == level) {
+			if (equation[i].token.operatr == DIVIDE) {
+				if (di < 0) {
+					di = i;
+					continue;
+				}
+				grouper = true;
+				for (len = i + 2; len < e1; len += 2) {
+					if (equation[len].level == level && equation[len].token.operatr != DIVIDE)
+						break;
+				}
+				len -= i;
+				if (edi == e1) {
+					i += len;
+					edi = i;
+					continue;
+				}
+				blt(scratch, &equation[i], len * sizeof(token_type));
+				blt(&equation[di+len], &equation[di], (i - di) * sizeof(token_type));
+				blt(&equation[di], scratch, len * sizeof(token_type));
+				edi += len;
+				i += len - 2;
+			} else {
+				if (di >= 0 && edi == e1)
+					edi = i;
+			}
+		}
+	}
+	if (!grouper) {
+		return;
+	}
+	for (i = di + 1; i < edi; i++) {
+		if (equation[i].level == level && equation[i].kind == OPERATOR) {
+			equation[i].token.operatr = TIMES;
+		}
+		equation[i].level++;
+	}
+}
 
 /*
- * Combine denominators in equation side.
+ * Group denominators together.
+ * Grouping here means converting "a/b/c/d*e" to "a*e/(b*c*d)" or "a/(b*c*d)*e",
+ * not guaranteed to put the grouped divisors last, reorder() puts divisors last.
+ */
+void
+group_proc(equation, np)
+token_type	*equation;
+int		*np;
+{
+	group_recurse(equation, np, 0, 1);
+}
+
+/*
+ * This function is the guts of the display command.
+ */
+void
+make_fractions_and_group(n)
+int	n;
+{
+	if (n_lhs[n] <= 0)
+		return;
+	elim_loop(lhs[n], &n_lhs[n]);
+	make_fractions(lhs[n], &n_lhs[n]);
+	group_proc(lhs[n], &n_lhs[n]);
+
+	if (n_rhs[n]) {
+		elim_loop(rhs[n], &n_rhs[n]);
+		make_fractions(rhs[n], &n_rhs[n]);
+		group_proc(rhs[n], &n_rhs[n]);
+	}
+}
+
+/*
+ * Combine fractions by making the denominators all the same.
  * This means converting "a/b+c/d" to "(a*d+c*b)/b/d".
  *
- * If start_flag is 0, only combine denominators to simplify complex fractions.
+ * If start_flag is 0, only combine denominators to convert complex fractions to simple fractions.
  *
  * If start_flag is 1, always combine denominators.
  *
@@ -335,97 +431,3 @@ do_gcd:
 		scratch[k].level++;
 	goto end_mess;
 }
-
-/*
- * This function is the guts of the "flist" command.
- */
-make_fractions_and_group(n)
-int	n;
-{
-	if (n_lhs[n] <= 0)
-		return;
-	elim_loop(lhs[n], &n_lhs[n]);
-	make_fractions(lhs[n], &n_lhs[n]);
-	group_proc(lhs[n], &n_lhs[n]);
-
-	if (n_rhs[n]) {
-		elim_loop(rhs[n], &n_rhs[n]);
-		make_fractions(rhs[n], &n_rhs[n]);
-		group_proc(rhs[n], &n_rhs[n]);
-	}
-}
-
-/*
- * Group denominators together.
- * Grouping here means converting "a/b/c" to "a/(b*c)".
- */
-group_proc(equation, np)
-token_type	*equation;
-int		*np;
-{
-	group_recurse(equation, np, 0, 1);
-}
-
-static void
-group_recurse(equation, np, loc, level)
-token_type	*equation;
-int		*np, loc, level;
-{
-	int		i;
-	int		len;
-	int		di, edi;
-	int		grouper = false;
-	int		e1;
-
-	for (i = loc; i < *np && equation[i].level >= level;) {
-		if (equation[i].level > level) {
-			group_recurse(equation, np, i, level + 1);
-			i++;
-			for (; i < *np && equation[i].level > level; i += 2)
-				;
-			continue;
-		}
-		i++;
-	}
-	e1 = i;
-	di = -1;
-	edi = e1;
-	for (i = loc + 1; i < e1; i += 2) {
-		if (equation[i].level == level) {
-			if (equation[i].token.operatr == DIVIDE) {
-				if (di < 0) {
-					di = i;
-					continue;
-				}
-				grouper = true;
-				for (len = i + 2; len < e1; len += 2) {
-					if (equation[len].level == level && equation[len].token.operatr != DIVIDE)
-						break;
-				}
-				len -= i;
-				if (edi == e1) {
-					i += len;
-					edi = i;
-					continue;
-				}
-				blt(scratch, &equation[i], len * sizeof(token_type));
-				blt(&equation[di+len], &equation[di], (i - di) * sizeof(token_type));
-				blt(&equation[di], scratch, len * sizeof(token_type));
-				edi += len;
-				i += len - 2;
-			} else {
-				if (di >= 0 && edi == e1)
-					edi = i;
-			}
-		}
-	}
-	if (!grouper) {
-		return;
-	}
-	for (i = di + 1; i < edi; i++) {
-		if (equation[i].level == level && equation[i].kind == OPERATOR) {
-			equation[i].token.operatr = TIMES;
-		}
-		equation[i].level++;
-	}
-}
