@@ -1,40 +1,13 @@
 /*
  * Mathomatic unfactorizing (expanding) routines.
  *
- * Copyright (C) 1987-2007 George Gesslein II.
+ * Copyright (C) 1987-2008 George Gesslein II.
  */
 
 #include "includes.h"
 
 static int	unf_sub();
 
-/*
- * Totally unfactor equation side and simplify.
- */
-uf_simp(equation, np)
-token_type	*equation;	/* pointer to beginning of equation side */
-int		*np;		/* pointer to length of equation side */
-{
-	if (*np == 0)
-		return;
-	uf_tsimp(equation, np);
-	uf_power(equation, np);
-	uf_repeat(equation, np);
-	uf_tsimp(equation, np);
-}
-
-/*
- * Unfactor equation side and simplify.
- * Don't call uf_repeat().
- */
-uf_simp_no_repeat(equation, np)
-token_type	*equation;
-int		*np;
-{
-	uf_power(equation, np);
-	uf_tsimp(equation, np);
-}
-
 /*
  * Unfactor times and divide only and simplify.
  */
@@ -54,22 +27,6 @@ int		*np;
 	return rv;
 }
 
-/*
- * Totally unfactor equation side with no simplification.
- */
-int
-ufactor(equation, np)
-token_type	*equation;
-int		*np;
-{
-	int	rv;
-
-	uf_repeat(equation, np);
-	rv = uf_times(equation, np);
-	uf_allpower(equation, np);
-	return rv;
-}
-
 /*
  * Unfactor power only.
  * (a * b)^c -> a^c * b^c
@@ -111,9 +68,10 @@ int		*np;
 }
 
 /*
- * Unfactor all power operators.  Same as a call to uf_pplus()
- * and uf_power(), only faster.
+ * Unfactor all power operators.
+ * Same as a call to uf_pplus() and uf_power(), only faster.
  */
+void
 uf_allpower(equation, np)
 token_type	*equation;
 int		*np;
@@ -124,11 +82,14 @@ int		*np;
 }
 
 /*
- * Unfactor power only if it will help with expansion.
+ * Unfactor power only if it will help with expansion and exponent is <= 100.
  * (a + 1)^2 -> (a + 1)*(a + 1)
+ * Afterwards we also sneak in simplifying division by irrational constants,
+ * because it works best here.
  *
- * uf_times() is usually called after this.
+ * uf_times() is usually called after this to complete the expansion.
  */
+void
 uf_repeat(equation, np)
 token_type	*equation;
 int		*np;
@@ -145,6 +106,7 @@ int		*np;
  * a^2 -> a*a
  * Useful for removing all integer powers.
  */
+void
 uf_repeat_always(equation, np)
 token_type	*equation;
 int		*np;
@@ -155,6 +117,51 @@ int		*np;
 	}
 }
 
+/*
+ * Totally unfactor equation side and simplify.
+ */
+void
+uf_simp(equation, np)
+token_type	*equation;	/* pointer to beginning of equation side */
+int		*np;		/* pointer to length of equation side */
+{
+	if (*np == 0)
+		return;
+	uf_tsimp(equation, np);
+	uf_power(equation, np);
+	uf_repeat(equation, np);
+	uf_tsimp(equation, np);
+}
+
+/*
+ * Unfactor equation side and simplify.
+ * Don't call uf_repeat().
+ */
+void
+uf_simp_no_repeat(equation, np)
+token_type	*equation;
+int		*np;
+{
+	uf_power(equation, np);
+	uf_tsimp(equation, np);
+}
+
+/*
+ * Totally unfactor equation side with no simplification.
+ */
+int
+ufactor(equation, np)
+token_type	*equation;
+int		*np;
+{
+	int	rv;
+
+	uf_repeat(equation, np);
+	rv = uf_times(equation, np);
+	uf_allpower(equation, np);
+	return rv;
+}
+
 static inline void
 no_divide(equation, np)
 token_type	*equation;
@@ -167,6 +174,7 @@ int		*np;
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == DIVIDE) {
 			level = equation[i].level;
+#if	false
 			if (partial_flag != 2) {
 				plus_flag = false;
 				for (j = i + 2; j < *np; j += 2) {
@@ -182,6 +190,7 @@ int		*np;
 				if (!plus_flag)
 					continue;
 			}
+#endif
 			for (j = i - 1; j >= 0; j--) {
 				if (equation[j].level < level)
 					break;
@@ -446,6 +455,7 @@ u2_again:
 			}
 		}
 do_repeat:
+		/* unfactor power: (a + 1)^2 -> (a + 1)*(a + 1) */
 		if (ii != 6 && ii != 8)
 			break;
 		i = loc;
@@ -454,7 +464,9 @@ do_repeat:
 		d1 = equation[i+1].token.constant;
 		if (!isfinite(d1) || d1 <= 1.0)
 			break;
-		if (ii != 8) {
+		if (ii != 8) {	/* if true, only do useful expansions */
+			if (d1 > 100.0)	/* ignore exponents over 100 */
+				break;
 			if ((i - b1) > 1 && d1 > 2.0 && fmod(d1, 1.0) != 0.0)
 				break;
 			if ((i - b1) == 1 && equation[b1].kind != CONSTANT)
@@ -556,7 +568,7 @@ int		*np;
 	return modified;
 }
 
-#if	false
+#if	false	/* the following is not currently used */
 /*
  * Convert a/(x^y) to a*(1/x)^y.
  * Return true if equation side is modified.
@@ -624,6 +636,7 @@ int		*np, i;
  * Convert anything times a negative constant to a positive constant
  * divided by -1.
  */
+void
 uf_neg_help(equation, np)
 token_type	*equation;
 int		*np;
@@ -660,7 +673,8 @@ int		*np;
 }
 
 /*
- * Convert 1/(k1^k2) to 1/(k1^(k2-1))/k1 if k1 is integer,
+ * Simplify division by irrational constants (roots like 2^.5) by
+ * converting 1/(k1^k2) to 1/(k1^(k2-1))/k1 if k1 is integer,
  * otherwise 1*((1/k1)^k2).
  *
  * Return true if equation side was modified.
